set -x

################## js-build-qtopia 0.1 ######################################
#
# I assume the build host is a linux on i386. Because of this assumption
# I included a pre-built i386/linux static binary version of uic and as
# a result you can skip building a bunch of craps.
#
# Additionally I assume you are building for ARM, which are reflected
# invarious config options.  It is possible to build for other arches
# once you take care all the cross paths and config options.
#
# We also assume you only build for the target.  As a result we don't
# do qvfb and other host tools.
#
# This has been tested with uclibc toolchain (buildroot).  Please let me if you
# have tested it on other platforms and any feedbacks in general.
#
# Jun Sun (jsun@junsun.net)
#
# TODO's:
# . The script assumes tslib is always needed.  We can make it more flexible so that
#   proper configuration is selected depending TSLIB is selected or not.
# 
# . We can make the script more robust by checking whether all the needed libraries
#   and headers are present.
##############################################################################

################## CONFIG ################

#
# This is the absolute path to top of your build directory, which is
# typically where this script lives
#
#BUILD_ROOT=/home/jsun/build/qtopia.new
BUILD_ROOT=`pwd`

#
# Absolution path of your cross compile tools
#
CROSS_COMPILE_PATH=/home/jsun/work/buildroot/buildroot/build_arm/staging_dir/bin
#CROSS_COMPILE_PATH=/home/jsun/build/oe/build-familiar/tmp/work/gcc-cross-3.4.4-r3/image/home/jsun/build/oe/build-familiar/tmp/cross/bin/
#CROSS_COMPILE_PATH=/home/jsun/build/buildroot/snapshot-20051122/buildroot/gcc342_build_arm_nofpu/staging_dir/bin/

#
# Do you need to build libuuid?  Set to 0 if your cross build environment
# already has libuuid *and* uuid/uuid.h file
#
NEED_UUID=1

#
# Do you need to build tslib?  Note this packages assumes QTE is built
# with tslib.  This option asks whether you want to build tslib with
# this script, or your cross build environment already has tslib built
# so that we can skip building it here.
#
NEED_TSLIB=1

#
# Do you want this script to build libjpeg?  Note this library is needed by 
# qtopia.  Tell the script whether this library is already supplied by your
# cross build environment.
#
NEED_JPEG=0

#
# Do you need to build QTE library?  Normally yes
#
NEED_QTE=1

#
# Do you need to build qtopia?  Normally yes.
# If you turn every thing off, you can source this file to get all the
# correct environment variables and then you can do line-by-line execution
# from shell.
#
NEED_QPE=1

#
# advanced options - changing them, you may break things
#
TRAGET_CFLAGS=
#TRAGET_CFLAGS="-O2"
#TARGET_CFLAGS="-march=armv5te -mtune=xscale -fexpensive-optimizations -fomit-frame-pointer -frename-registers -O2"
CROSS_COMPILE_PREFIX=${CROSS_COMPILE_PATH}/arm-linux-

################## CONSTANTS ##############
DOWNLOAD_DIR=${BUILD_ROOT}/download
BUILD_DIR=${BUILD_ROOT}/build
OUTPUT_DIR=${BUILD_ROOT}/output

E2FSPROGS_SRC="e2fsprogs-1.38.tar.gz"
E2FSPROGS_DIR="$BUILD_DIR/e2fsprogs-1.38"
E2FSPROGS_URL="http://dl.sourceforge.net/sourceforge/e2fsprogs/$E2FSPROGS_SRC"

TSLIB_SRC="mv-tslib-20030303.tar.gz"
TSLIB_DIR="$BUILD_DIR/tslib"
TSLIB_URL="http://linux.junsun.net/packages/qtopia/${TSLIB_SRC}"

JPEG_SRC="jpegsrc.v6b.tar.gz"
JPEG_DIR="$BUILD_DIR/jpeg-6b"
JPEG_URL="ftp://ftp.uu.net/graphics/jpeg/${JPEG_SRC}"

QTE_SRC="qt-embedded-2.3.10-free.tar.gz"
QTE_DIR="$BUILD_DIR/qt-2.3.10"
QTE_URL="ftp://ftp.trolltech.com/qt/source/${QTE_SRC}"

QPE_SRC="qtopia-free-source-2.1.1.tar.bz2"
QPE_DIR="$BUILD_DIR/qtopia-free-2.1.1"
QPE_URL="ftp://ftp.trolltech.com/qtopia/source/${QPE_SRC}"

################## BASICS ##############
mkdir -p $DOWNLOAD_DIR $BUILD_DIR $OUTPUT_DIR || (echo "ERROR!" ; exit 1)

################## TEST HOST ##############
wget --version 2>&1 > /dev/null || (echo "ERROR: You need wget"; exit 1)
strip --version 2>&1 > /dev/null || (echo "ERROR: You need strip"; exit 1)
tar --version 2>&1 > /dev/null || (echo "ERROR: You need tar"; exit 1)


################## BUILD ###############

# --- build uuid (from e2fsprogs)

if [ $NEED_UUID == 1 ]; then

cd $DOWNLOAD_DIR;

if [ ! -f $E2FSPROGS_SRC ];  then
	echo wget $E2FSPROGS_URL ...
	wget $E2FSPROGS_URL  || exit 1
fi

cd $BUILD_DIR

rm -rf $E2FSPROGS_DIR
tar xzf $DOWNLOAD_DIR/$E2FSPROGS_SRC	|| exit 1

cd $E2FSPROGS_DIR	|| exit 1

./configure --build=i386-unknown-linux-gnu --target=arm-unknown-linux-gnu --host=arm-unknown-linux-gnu --with-cc="${CROSS_COMPILE_PREFIX}gcc ${TARGET_CFLAGS}" --with-linker=${CROSS_COMPILE_PREFIX}ld --prefix=$OUTPUT_DIR || exit 1

make -C $E2FSPROGS_DIR/lib/uuid/ || exit 1

make -C $E2FSPROGS_DIR/lib/uuid/ install || exit 1

fi

# --- build tslib

if [ $NEED_TSLIB == 1 ]; then

cd $DOWNLOAD_DIR;

if [ ! -f $TSLIB_SRC ];  then
	echo wget $TSLIB_URL ...
	wget $TSLIB_URL  || exit 1
fi

cd $BUILD_DIR

rm -rf $TSLIB_DIR
tar xzf $DOWNLOAD_DIR/$TSLIB_SRC || exit 1

cd $TSLIB_DIR

tar xzf tslib-20030303.tar.gz  || exit 1
./dopatch

cd tslib-20030303

# [JSUN] all the fancy cross config does not seem to work! *ick*
#./configure --build=i386-unknown-linux-gnu --target=arm-unknown-linux-gnu --host=arm-unknown-linux-gnu --with-cc="${CROSS_COMPILE_PREFIX}gcc ${TARGET_CFLAGS}" --with-linker=${CROSS_COMPILE_PREFIX}ld --prefix=$OUTPUT_DIR || exit 1
./configure --prefix=$OUTPUT_DIR || exit 1

# we have to fix up the libtool
cp libtool libtool.orig
cat libtool.orig | \
	sed -e "s#CC=\"gcc\"#CC=\"${CROSS_COMPILE_PREFIX}gcc ${TARGET_CFLAGS}\"#" | \
	sed -e "s#LD=\"/usr/bin/ld\"#CC=\"${CROSS_COMPILE_PREFIX}gcc\"#" > \
	libtool

make CC="${CROSS_COMPILE_PREFIX}gcc ${TARGET_CFLAGS}" LD="${CROSS_COMPILE_PREFIX}gcc" || exit 1
make install  || exit 1

#
# this tslib is really weired.  There is gotta be a better package out there
# please let me know.  jsun@junsun.net
#
# IN our case, we need to use gcc to do the link because some options
# are passed in as -Wl (which my understanding is to pass linker options
# through gcc
#
fi

# --- build jpeg lib

if [ $NEED_JPEG == 1 ]; then

cd $DOWNLOAD_DIR;

if [ ! -f $JPEG_SRC ];  then
        echo wget $JPEG_URL ...
	        wget $JPEG_URL  || exit 1
	fi

cd $BUILD_DIR

rm -rf $JPEG_DIR
tar xzf $DOWNLOAD_DIR/$JPEG_SRC || exit 1

cd $JPEG_DIR

#./configure --enable-shared  --build=i386-unknown-linux-gnu --target=arm-unknown-linux-gnu --host=arm-unknown-linux-gnu --with-cc="${CROSS_COMPILE_PREFIX}gcc ${TARGET_CFLAGS}" --with-linker=${CROSS_COMPILE_PREFIX}ld --prefix=$OUTPUT_DIR || exit 1
./configure --build=i386-unknown-linux-gnu --target=arm-unknown-linux-gnu --host=arm-unknown-linux-gnu --with-cc="${CROSS_COMPILE_PREFIX}gcc ${TARGET_CFLAGS}" --with-linker=${CROSS_COMPILE_PREFIX}ld --prefix=$OUTPUT_DIR || exit 1

(make CC="${CROSS_COMPILE_PREFIX}gcc ${TARGET_CFLAGS}" LD="${CROSS_COMPILE_PREFIX}gcc" 2>&1 ) || exit 1
make install-lib

fi

# --- build qte & qtopia

# download them if necessary
cd $DOWNLOAD_DIR;
if [ ! -f $QTE_SRC ];  then
	echo wget $QTE_URL ...
	wget $QTE_URL  || exit 1
fi
if [ ! -f $QPE_SRC ];  then
	echo wget $QPE_URL ...
	wget $QPE_URL  || exit 1
fi

# qtopia/qte assume tools are in path
export PATH=${CROSS_COMPILE_PATH}:$PATH

# some common environment setups
export QTEDIR=$QTE_DIR
export QPEDIR=$QPE_DIR
export QTDIR=$QTEDIR

cd $BUILD_DIR

# untar the source
if [ $NEED_QPE == 1 ]; then
rm -rf $QPE_DIR
tar xjf $DOWNLOAD_DIR/$QPE_SRC || exit 1
fi

if [ $NEED_QTE == 1 ]; then

rm -rf $QTE_DIR
tar xzf $DOWNLOAD_DIR/$QTE_SRC || exit 1


# build QTE
cd $QTE_DIR

# I guess qtopia has a specifial expectations for qte
cp $QPE_DIR/src/qt/qconfig-qpe.h src/tools/

# configure; see ./configure -help -verbose
# you can replace -debug with -release
echo "yes" | \
	./configure -xplatform linux-arm-g++ -system-jpeg -no-xft -debug -tslib -qconfig qpe -depths 4,8,16,32 -no-qvfb -keypad-mode -I${OUTPUT_DIR}/include -L${OUTPUT_DIR}/lib  || \
	exit 1

# NOTE: if you failed at generating allmoc.cpp, you need to do a make clean
# and re-make again
(make sub-src 2>&1) || exit 1

## install qte

mkdir -p $OUTPUT_DIR/opt/qtopia/lib
cp -a lib/* $OUTPUT_DIR/opt/qtopia/lib/

fi

# --- make qpe ---

if [ $NEED_QPE == 1 ]; then
# copy the statically linked uic
cd $BUILD_DIR
cp $DOWNLOAD_DIR/uic $QTEDIR/bin

#patch qtopia
patch -p0 < $DOWNLOAD_DIR/jsun-linux-arm-qtopia-free-2.1.1.patch || exit 1

# not sure if this is necessary, actually
export LD_LIBRARY_PATH=$OUTPUT_DIR/lib:$LD_LIBRARY_PATH

cd $QPEDIR

# configure; 
./configure -xplatform linux-arm-g++ -debug -edition pda -no-qvfb -no-qtopiadesktop -I${OUTPUT_DIR}/include -luuid -L${OUTPUT_DIR}/lib -R${OUTPUT_DIR}/lib -prefix $OUTPUT_DIR/opt/qtopia || exit 1

# make
make 2>&1 || exit 1
make install || exit 1

fi

########################################################################
# just to be a little friendier if you source this file 
set +x
